@@ -6,6 +6,7 @@ import numpy as np
 import logging
 import getpass
 import json
+import uuid
 from askdata.insight import Insight
 from askdata.channel import Channel
 from askdata.catalog import Catalog
@@ -19,8 +20,6 @@ from IPython.core.display import display, HTML
 
 if TYPE_CHECKING:
     from askdata.askdata_client import Askdata
-import re
-from datetime import datetime
 
 _LOG_FORMAT = "[%(filename)s:%(lineno)s - %(funcName)20s() ] - %(asctime)s --> %(message)s"
 g_logger = logging.getLogger()
@@ -841,6 +840,99 @@ class Askdata(SignUp):
     def get_token(self, username, password):
         return self._token
 
+    def save_dataset(self, df:pd.DataFrame, workspace: str, replace=False, dataset_name="", locale="en"):
+
+        if "/" in workspace:
+            agentSlug = workspace.split("/")[0]
+            datasetSlug = workspace.split("/")[1]
+        else:
+            agentSlug = workspace
+            datasetSlug = "parquet_" + uuid.uuid4().__str__()
+
+        url = self.base_url + "/smartbot/agents/{}/datasets/{}/data_table".format(agentSlug, datasetSlug)
+
+        fields = []
+        for column in df.columns:
+            fields.append({"name": column})
+
+        schema = {"fields": fields}
+
+        body = {
+            "data": df.to_dict(orient="record"),
+            "schema": schema,
+            "locale": locale
+        }
+
+        if (replace):
+
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            response = s.put(url=url, json=body, headers=headers)
+            response.raise_for_status()
+            dataset = response.json()
+            dataset_id = dataset["id"]
+
+        else:
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            response = s.post(url=url, json=body, headers=headers)
+            response.raise_for_status()
+            dataset = response.json()
+            dataset_id = dataset["id"]
+
+        if (dataset_name != ""):
+            url = self.base_url + "/smartbot/agents/" + dataset["agents"][0] + "/datasets/" + dataset_id + "/settings"
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            response = s.get(url=url, headers=headers)
+            response.raise_for_status()
+            settings = response.json()
+
+            settings["label"] = dataset_name
+
+            s = requests.Session()
+            s.keep_alive = False
+            retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
+            s.mount('https://', HTTPAdapter(max_retries=retries))
+
+            headers = {
+                "Content-Type": "application/json",
+                "Authorization": "Bearer" + " " + self._token
+            }
+            response = s.put(url=url, json=settings, headers=headers)
+            response.raise_for_status()
+
+        # Execute sync
+        headers = {
+            "Content-Type": "application/json",
+            "Authorization": "Bearer" + " " + self._token
+        }
+        dataset_url = self.base_url + '/smartdataset/datasets/' + dataset_id + '/sync'
+        r = requests.post(url=dataset_url, headers=headers)
+        r.raise_for_status()
+
+
     def join(self, df:pd.DataFrame, with_query:str, workspace:str, how="inner", on=None, left_on=None, right_on=None, lang="en"):
 
         s = requests.Session()
