@@ -132,7 +132,6 @@ class AskdataPandasExtension:
             retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
             s.mount('https://', HTTPAdapter(max_retries=retries))
 
-            print(url)
             headers = {
                 "Content-Type": "application/json",
                 "Authorization": "Bearer" + " " + self._token
@@ -163,7 +162,7 @@ class AskdataPandasExtension:
         dataset_url = askdata_url + '/smartdataset/datasets/' + dataset_id + '/sync'
         r = requests.post(url=dataset_url, headers=headers)
         r.raise_for_status()
-        return r
+
 
     def join(self, with_query:str, workspace:str, how="inner", on=None, left_on=None, right_on=None, lang="en"):
 
